/*
 * This file is part of Freedom-Network-Suite - https://github.com/AtlasMediaGroup/Freedom-Network-Suite
 * Copyright (C) 2023 Total Freedom Server Network and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package fns.datura.cmd;

import fns.datura.Datura;
import fns.patchwork.base.Shortcuts;
import fns.patchwork.command.Commander;
import fns.patchwork.command.annotation.Completion;
import fns.patchwork.command.annotation.Info;
import fns.patchwork.command.annotation.Permissive;
import fns.patchwork.command.annotation.Subcommand;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;
import org.jetbrains.annotations.NotNull;

@Info(name = "halt", description = "Halt a single player, or every player.", usage = "/<command> <player | all> <on, off>")
@Permissive(perm = "datura.halt")
@Completion(index = 0, args = {"%player%", "all"})
@Completion(index = 1, args = {"on", "off"})
public class HaltCommand extends Commander
{
    private final Datura plugin = Shortcuts.provideModule(Datura.class);

    /**
     * Initializes this command object. The provided {@link JavaPlugin} should be the plugin which contains the
     * command.
     * <p>
     * This constructor will automatically register all subcommands and completions for this command. It will also
     * automatically infer all required information from the provided {@link Info} and {@link Permissive} annotations.
     *
     * @param plugin The plugin which contains this command.
     */
    public HaltCommand(@NotNull final JavaPlugin plugin)
    {
        super(plugin);
    }

    @Subcommand(permission = "datura.halt", args = {Player.class, String.class})
    public void haltPlayer(final CommandSender sender, final Player target, final String toggle)
    {
        if (toggle.equalsIgnoreCase("on"))
        {
            plugin.getHalter()
                  .halt(target.getUniqueId());

            target.sendPlainMessage("You have been frozen!");
            sender.sendPlainMessage("You have halted " + target.getName() + ".");
        } else if (toggle.equalsIgnoreCase("off"))
        {
            plugin.getHalter()
                  .stop(target.getUniqueId());

            target.sendPlainMessage("You have been unfrozen!");
            sender.sendPlainMessage("You have unhalted " + target.getName() + ".");
        }
    }

    @Subcommand(permission = "datura.halt.all", args = {String.class, String.class})
    public void haltAll(final CommandSender sender, final String all, final String toggle)
    {
        if (all.equalsIgnoreCase("all"))
        {
            if (toggle.equalsIgnoreCase("on"))
            {
                Bukkit.getServer()
                      .getOnlinePlayers()
                      .forEach(player -> plugin.getHalter()
                                               .halt(player.getUniqueId()));

                final Component message = sender.name()
                                                .append(Component.text(": Freezing all players"))
                                                .color(NamedTextColor.AQUA);

                Bukkit.broadcast(message);
                sender.sendPlainMessage("All players have been halted.");
            } else if (toggle.equalsIgnoreCase("off"))
            {
                plugin.getHalter()
                      .clear();

                Bukkit.broadcast(Component.text("All players have been unfrozen!", NamedTextColor.AQUA));
                sender.sendPlainMessage("All players have been unhalted.");
            }
        }
    }
}
